/*******************************************************************************
* File Name: SCB_1_I2C.c
* Version 1.0
*
* Description:
*  This file provides the source code to the API for the SCB Component in
*  I2C mode.
*
* Note:
*
*******************************************************************************
* Copyright 2013, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "SCB_1_PVT.h"
#include "SCB_1_I2C_PVT.h"


/***************************************
*      I2C Private Vars
***************************************/

volatile uint8 SCB_1_state;  /* Current state of I2C FSM */


#if(SCB_1_SCB_MODE_UNCONFIG_CONST_CFG)

    /***************************************
    *  Config Structure Initialization
    ***************************************/

    /* Constant configuration of I2C */
    const SCB_1_I2C_INIT_STRUCT SCB_1_configI2C =
    {
        SCB_1_I2C_MODE,
        SCB_1_I2C_OVS_FACTOR_LOW,
        SCB_1_I2C_OVS_FACTOR_HIGH,
        SCB_1_I2C_MEDIAN_FILTER_ENABLE,
        SCB_1_I2C_SLAVE_ADDRESS,
        SCB_1_I2C_SLAVE_ADDRESS_MASK,
        SCB_1_I2C_ACCEPT_ADDRESS,
        SCB_1_I2C_WAKE_ENABLE
    };

    /*******************************************************************************
    * Function Name: SCB_1_I2CInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for I2C operation.
    *
    * Parameters:
    *  config:  Pointer to a structure that contains the following ordered list of
    *           fields. These fields match the selections available in the
    *           customizer.
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_1_I2CInit(const SCB_1_I2C_INIT_STRUCT *config)
    {
        if(NULL == config)
        {
            CYASSERT(0u != 0u); /* Halt execution due bad fucntion parameter */
        }
        else
        {
            /* Configure pins */
            SCB_1_SetPins(SCB_1_SCB_MODE_I2C, SCB_1_DUMMY_PARAM, 
                                                                    SCB_1_DUMMY_PARAM);

            /* Store internal configuration */
            SCB_1_scbMode       = (uint8) SCB_1_SCB_MODE_I2C;
            SCB_1_scbEnableWake = (uint8) config->enableWake;

            SCB_1_mode          = (uint8) config->mode;
            SCB_1_acceptAddr    = (uint8) config->acceptAddr;

            /* Configure I2C interface */
            SCB_1_CTRL_REG     = SCB_1_GET_CTRL_ADDR_ACCEPT(config->acceptAddr) |
                                            SCB_1_GET_CTRL_EC_AM_MODE (config->enableWake);

            SCB_1_I2C_CTRL_REG = SCB_1_GET_I2C_CTRL_HIGH_PHASE_OVS(config->oversampleHigh) |
                                            SCB_1_GET_I2C_CTRL_LOW_PHASE_OVS (config->oversampleLow)  |
                                            SCB_1_GET_I2C_CTRL_SL_MSTR_MODE  (config->mode)           |
                                            SCB_1_I2C_CTRL;

            /* Adjust SDA filter settigns */
            SCB_1_SET_I2C_CFG_SDA_FILT_TRIM(SCB_1_EC_AM_I2C_CFG_SDA_FILT_TRIM);


            /* Configure RX direction */
            SCB_1_RX_CTRL_REG      = SCB_1_GET_RX_CTRL_MEDIAN(config->enableMedianFilter) |
                                                SCB_1_I2C_RX_CTRL;
            SCB_1_RX_FIFO_CTRL_REG = SCB_1_CLEAR_REG;

            /* Set default address and mask */
            SCB_1_RX_MATCH_REG    = ((SCB_1_I2C_SLAVE) ?
                                                (SCB_1_GET_I2C_8BIT_ADDRESS(config->slaveAddr) |
                                                 SCB_1_GET_RX_MATCH_MASK(config->slaveAddrMask)) :
                                                (SCB_1_CLEAR_REG));


            /* Configure TX direction */
            SCB_1_TX_CTRL_REG      = SCB_1_I2C_TX_CTRL;
            SCB_1_TX_FIFO_CTRL_REG = SCB_1_CLEAR_REG;

            /* Configure interrupt sources */
            SCB_1_INTR_I2C_EC_MASK_REG = SCB_1_NO_INTR_SOURCES;
            SCB_1_INTR_SPI_EC_MASK_REG = SCB_1_NO_INTR_SOURCES;
            SCB_1_INTR_RX_MASK_REG     = SCB_1_NO_INTR_SOURCES;
            SCB_1_INTR_TX_MASK_REG     = SCB_1_NO_INTR_SOURCES;

            SCB_1_INTR_SLAVE_MASK_REG  = ((SCB_1_I2C_SLAVE) ?
                                                     (SCB_1_I2C_INTR_SLAVE_MASK) :
                                                     (SCB_1_CLEAR_REG));

            SCB_1_INTR_MASTER_MASK_REG = ((SCB_1_I2C_MASTER) ?
                                                     (SCB_1_I2C_INTR_MASTER_MASK) :
                                                     (SCB_1_CLEAR_REG));

            /* Configure interrupt with I2C handler and enabled it */
            SCB_1_SCB_IRQ_StartEx(&SCB_1_I2C_ISR);


            /* Configure global variables */
            SCB_1_state = SCB_1_I2C_FSM_IDLE;

            /* Internal slave variable */
            SCB_1_slStatus        = 0u;
            SCB_1_slRdBufIndex    = 0u;
            SCB_1_slWrBufIndex    = 0u;
            SCB_1_slOverFlowCount = 0u;

            /* Internal master variable */
            SCB_1_mstrStatus     = 0u;
            SCB_1_mstrRdBufIndex = 0u;
            SCB_1_mstrWrBufIndex = 0u;
        }
    }

#else

    /*******************************************************************************
    * Function Name: SCB_1_I2CInit
    ********************************************************************************
    *
    * Summary:
    *  Configures the SCB for I2C operation.
    *
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    *******************************************************************************/
    void SCB_1_I2CInit(void)
    {
        /* Configure I2C interface */
        SCB_1_CTRL_REG     = SCB_1_I2C_DEFAULT_CTRL;
        SCB_1_I2C_CTRL_REG = SCB_1_I2C_DEFAULT_I2C_CTRL;

        /* Adjust SDA filter settigns */
        SCB_1_SET_I2C_CFG_SDA_FILT_TRIM(SCB_1_EC_AM_I2C_CFG_SDA_FILT_TRIM);

        /* Configure RX direction */
        SCB_1_RX_CTRL_REG      = SCB_1_I2C_DEFAULT_RX_CTRL;
        SCB_1_RX_FIFO_CTRL_REG = SCB_1_I2C_DEFAULT_RX_FIFO_CTRL;

        /* Set default address and mask */
        SCB_1_RX_MATCH_REG     = SCB_1_I2C_DEFAULT_RX_MATCH;

        /* Configure TX direction */
        SCB_1_TX_CTRL_REG      = SCB_1_I2C_DEFAULT_TX_CTRL;
        SCB_1_TX_FIFO_CTRL_REG = SCB_1_I2C_DEFAULT_TX_FIFO_CTRL;

        /* Configure interrupt sources */
        SCB_1_INTR_I2C_EC_MASK_REG = SCB_1_I2C_DEFAULT_INTR_I2C_EC_MASK;
        SCB_1_INTR_SPI_EC_MASK_REG = SCB_1_I2C_DEFAULT_INTR_SPI_EC_MASK;
        SCB_1_INTR_SLAVE_MASK_REG  = SCB_1_I2C_DEFAULT_INTR_SLAVE_MASK;
        SCB_1_INTR_MASTER_MASK_REG = SCB_1_I2C_DEFAULT_INTR_MASTER_MASK;
        SCB_1_INTR_RX_MASK_REG     = SCB_1_I2C_DEFAULT_INTR_RX_MASK;
        SCB_1_INTR_TX_MASK_REG     = SCB_1_I2C_DEFAULT_INTR_TX_MASK;

        /* Configure interrupt with I2C handler and enabled it */
        SCB_1_SCB_IRQ_StartEx(&SCB_1_I2C_ISR);


        /* Configure global variables */
        SCB_1_state = SCB_1_I2C_FSM_IDLE;

        #if(SCB_1_I2C_SLAVE)
            /* Internal slave variable */
            SCB_1_slStatus        = 0u;
            SCB_1_slRdBufIndex    = 0u;
            SCB_1_slWrBufIndex    = 0u;
            SCB_1_slOverFlowCount = 0u;
        #endif /* (SCB_1_I2C_SLAVE) */

        #if(SCB_1_I2C_MASTER)
        /* Internal master variable */
            SCB_1_mstrStatus     = 0u;
            SCB_1_mstrRdBufIndex = 0u;
            SCB_1_mstrWrBufIndex = 0u;
        #endif /* (SCB_1_I2C_MASTER) */
    }
#endif /* (SCB_1_SCB_MODE_UNCONFIG_CONST_CFG) */


/*******************************************************************************
* Function Name: SCB_1_I2CStop
********************************************************************************
*
* Summary:
*  Initializes I2C registers with initial values provided from customizer.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global variables:
*
*
*******************************************************************************/
void SCB_1_I2CStop(void)
{
    SCB_1_state = SCB_1_I2C_FSM_IDLE;
}


/*******************************************************************************
* Function Name: SCB_1_I2CSaveConfig
********************************************************************************
*
* Summary:
*  Wakeup disabled: does nothing.
*  Wakeup  enabled: clears SCB_backup.enableState and enables
*  SCB_INTR_I2C_EC_WAKE_UP interrupt used for wakeup. This interrupt source
*  tracks in active mode therefore it does not require to be cleared.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SCB_1_I2CSaveConfig(void)
{
    #if(SCB_1_CHECK_I2C_WAKE_ENABLE)
    {
        SCB_1_backup.enableState = 0u; /* Keep SCB enabled */

        /* Enable interrupt to wakeup the device */
        SCB_1_SetI2CExtClkInterruptMode(SCB_1_INTR_I2C_EC_WAKE_UP);
    }
    #endif
}


/*******************************************************************************
* Function Name: SCB_1_I2CRestoreConfig
********************************************************************************
*
* Summary:
*  Does nothing.
*
* Parameters:
*  None
*
* Return:
*  None
*
*******************************************************************************/
void SCB_1_I2CRestoreConfig(void)
{
    /* Does nothing: as wake is masked-off in the interrupt */
}


/* [] END OF FILE */
